#!/usr/local/bin/python
#
# $Id: //projects/discord/discord.py#2 $ $Date: 2006/07/07 $

"""
A module for doing Gregorian -> Discordian date conversions.
"""

# Discordianism is stupid.

__program__ = 'discord'
__version__ = '1.0.1'
__url__ = 'http://www.alcyone.com/pyos/discord/'
__author__ = 'Erik Max Francis <max@alcyone.com>'
__copyright__ = 'Copyright (C) 2003 Erik Max Francis'
__license__ = 'GPL'


import time


SEASONS = ['Chaos', 'Discord', 'Confusion', 'Bureaucracy', 'Aftermath']
SEASONS_ABBR = ['Cha', 'Dis', 'Con', 'Bur', 'Aft']
WEEKDAYS = ['Sweetmorn', 'Boomtime', 'Pungenday', 'Prickle-Prickle', 'Setting Orange']
WEEKDAYS_ABBR = ['Sm', 'Bt', 'Pd', 'Pp', 'So']
APOSTLE_HOLIDAYS = ['Mungday', 'Mojoday', 'Syaday', 'Zaraday', 'Maladay']
SEASON_HOLIDAYS = ['Chaoflux', 'Discoflux', 'Confuflux', 'Bureflux', 'Afflux']
ST_TIBS_DAY = "St. Tib's Day"
ST_TIBS_DAY_ABBR = 'ST.'



def discord(info):

    """Given a 9-tuple returned from time.localtime, etc., Return a
    5-tuple consisting of the year, season, day, weekday, and a
    Boolean indicating whether or not it's St. Tib's day."""

    tibs = False
    count = info[7] - 1
    y = info[0]
    leap = not (y%4) and ((y%100) or not (y%400))
    if leap:
        if count + 1 == 60:
            tibs = True
        elif count + 1 > 60:
            count -= 1
    assert info[0] >= 100 # don't handle two-digit years
    year = 1166 + info[0]
    season, day = divmod(count, 73)
    day += 1
    weekday = count%5
    return year, season, day, weekday, tibs

def string(data, abbreviations=True, holidays=True):

    """Given a 5-tuple of information about the Discordian date,
    return a string representing it.  St. Tib's Day is represented by
    'ST.' and the year."""

    year, season, day, weekday, tibs = data
    if tibs:
        if abbreviations:
            return "%s %d" % (ST_TIBS_DAY_ABBR, year)
        else:
            return "%s, %d" % (ST_TIBS_DAY, year)
    if abbreviations:
        result = ("%s, %s %s %s" % 
                  (WEEKDAYS_ABBR[weekday], year, SEASONS_ABBR[season], day))
    else:
        result = ("%s, %s %s %s" % 
                  (WEEKDAYS[weekday], year, SEASONS[season], day))
    if holidays:
        if day == 5:
            result += " (%s)" % APOSTLE_HOLIDAYS[season]
        if day == 50:
            result += " (%s)" % SEASON_HOLIDAYS[season]
    return result
    

def main():
    now = time.time()
    info = time.localtime(now)
    print string(discord(info))

if __name__ == '__main__': main()
